#!/usr/bin/perl -w

sub usage(){
    print "Usage: $0 -skd <skd file> [-st <id>] [-dir <a[:b:c:d]>] [-fn <n>]\n";
    print " \t [-sim <n>] [-exe] \n";
    print " Auto Observation support tool for recording with ADS2000+PC-VSI\n";
    print " -st: Station ID used in the sched file , Default is 'O' \n";
    print " -dir: Output data dirs, if less than four are specified, all the data is stored on the same one directory\n";
    print " -fn: File naming convention: (default 1)\n";
    print "     1 : XDDDNNNN.dat (default)\n";
    print "    -1 : XDDDNNNN.#ch.dat \n";
    print "     2 : sidDDDHHMMSSG.dat \n";
    print "    -2 : sidDDDHHMMSSG.#ch.dat\n";
    print "     3 : expid_sidG_scanid_YYYYDDDHHMMSS.k5 \n";
    print " -sim: Simulation mode, 0: Not Simlulation\n";
    print "     1: Assuming Now the 5 sec before start time \n";
    print "     2: No Waiting.\n";
    print " -----------------------------------------\n";
    exit(1);
}

use lib "$ENV{HOME}/prog/perl/module";
use Sched_db;
use Time_lib;
use Term::ANSIScreen qw/:color :cursor :screen :keyboard/;
use Time::HiRes qw(clock_gettime clock_getres CLOCK_REALTIME usleep);

# k5vsi-vssp-dim -m '08Msps/2bit/04ch' -o test-vssp.dat1:test-vssp.dat2:test-vssp.dat3:test-vssp.dat4 -g 0:1:2:3 -t 10s -d

my $cmd0 = "k5vsi-vssp-dim";

{
    usage() if(@ARGV < 1); 
    my ($skdfile, $stid);
    my @odir = qw(. . . .);
    my $exflag =0;
    my $fn=1;
    my $sim = 0;
    while(@ARGV){
	my $opt = shift;
	if($opt =~/-skd/){
	    unless(@ARGV){ usage();}
	    $skdfile = shift;
	}elsif($opt =~/-st/){
	    unless(@ARGV){ usage();}
	    $stid = shift;
	}elsif($opt =~/-dir/){
	    unless(@ARGV){ usage();}
	    my $odir = shift;
	    my @terms = split(/:/,$odir);
	    if( @terms < 4){ 
		@odir = ($terms[0],$terms[0],$terms[0],$terms[0]);
	    }else{@odir = @terms; }
	}elsif($opt =~/-fn/){
	    unless(@ARGV){ usage();}
	    $fn = shift;
	}elsif($opt =~/-sim/){
	    unless(@ARGV){ usage();}
	    $sim = shift;
	}elsif($opt =~/-exe/){
	    $exflag=1;
	}
    }

    unless( defined($skdfile)){
	print " Error Missing sched file.!!\n";
	usage();
    }
    my $skd = new Sched_db;
    $skd->Read($skdfile);
    my $nskd = $skd->Nskd();
    my $srate = $skd->Srate();
    my $expcode = $skd->Expr();
    my $nch = 4;
    my $opt0 = sprintf("-m '%02dMsps/2bit/%02dch' -g 0:1:2:3",
		       $srate, $nch);
    my $catch_flag=0;
    for(my $iscan=0; $iscan < $nskd; $iscan++){
	my $srcname = $skd->Skd_src($iscan);
	my $dur = $skd->Skd_dur($iscan,$stid);
	next if( $dur < 0);
	$t1 = $skd->Skd_start($iscan,3);
	$t2 = $skd->Skd_stop($iscan,3);
	my $tnow = clock_gettime(CLOCK_REALTIME);
	my $twait = $t1->UXtime() - $tnow;
	my $twait2 = $t2->UXtime() - $tnow;
#	my $twait2 = $t2->SecFromNow();
	if($sim == 1){ $twait = 5;
	}elsif( $sim == 2){ $twait =0;}
	if( $twait > 1){
	    print "\n\n\n" if ($catch_flag==0);
	    $catch_flag=1;
	    my $tm = new Time_lib;
	    while($twait > 0.5){
		print up(1);
		my $tw = $twait > 1 ? 1.e6 : ($twait-0.2)*1.e6;
		$tm->Set_time_now();
		printf "Now(%s) waiting (%d sec) for %s\n",
			$tm->Str_fmt3(2), $twait,$t1->Str_fmt3(2);
		usleep($tw);
		$twait = $t1->UXtime() - clock_gettime(CLOCK_REALTIME);
	    }
	}elsif($twait2 < 0){
	    print "$iscan ";
	    next;
	}
	my @fname;
	for(my $igrp=0; $igrp<4; $igrp++){
	    $fname[$igrp] = $odir[$igrp] . "/"
		. &Naming($skd,$iscan,$stid,$igrp,$fn);
	}
	my $opt1 = sprintf("-t %ds -o %s:%s:%s:%s", 
			  $dur, $fname[0],$fname[1],$fname[2],$fname[3]);
	my $cmd = "$cmd0 $opt0 $opt1 ";
	print " $cmd\n";
	if($exflag){
	    system($cmd);
	}
    }
    
}
    


sub Naming($$$;$){
    my ($skd, $i, $id, $igrp,$fn) = @_;
    $fn = 1 unless(defined($fn));
    my $t1 = $skd->Skd_start($i,3);
    my $doy = $t1->Doy();
    my @grp = qw(a b c d);
    my @ydhms = $t1->YDHms();
    my $fname;    
    if($fn == 1){
	$fname = sprintf("%s%03d%04d.dat",$id,$doy,$i);
    }elsif($fn == -1){
	$fname = sprintf("%s%03d%04d.%d.dat",$id,$doy,$igrp,$i);
    }elsif($fn == 2){
	$fname = sprintf("%s%03d%02d%02d%02d%s.dat",
			 $id,$ydhms[1],$ydhms[2],$ydhms[3],$ydhms[4],
			 $grp[$igrp]);
    }elsif($fn == -2){
	$fname = sprintf("%s%03d%02d%02d%02d%s.%d.dat",
			 $id,$ydhms[1],$ydhms[2],$ydhms[3],$ydhms[4],
			 $grp[$igrp], $igrp);
    }elsif($fn == 3){
	my $expr = $skd->Expr();
	my $src =  $skd->Skd_src($i);
	$fname = sprintf("%s_%s%s_%s_%04d%03d%02d%02d%02d.k5",
			 $expr, $id, $grp[$igrp],$src,$ydhms[0],
			 $ydhms[1],$ydhms[2],$ydhms[3],$ydhms[4]);
    }
    return $fname;
}
