/** 
 * @Author M.Sekido
 * @Version $Rev: 208 $
 * @Date  $Date: 2016-09-30 08:33:01 +0000 (Fri, 30 Sep 2016) $
 * $Id: sudp_send.cpp 208 2016-09-30 08:33:01Z vlbi $ 
 */
#include <iostream>
#include <vector>
#include <string>

#include <cstdio>
#include <cstring>
#include <cstdlib>
using namespace std;

#include "SUDPProto.hpp"
#include "strngutl.hpp"
#include "VLBI_Frame.hpp"
#include "VDIF_Frame.hpp"
#include "InFile_mdl.hpp"
#include "InMem_mdl.hpp"
#include "Trans_ctl.hpp"
#include "StaticVals.hpp"

extern "C"{
#include <signal.h>
}
const char* version_str = "$Rev: 208 $ updated $Date: 2016-09-30 08:33:01 +0000 (Fri, 30 Sep 2016) $ ";
char *me;  // My name

void usage(){
  cerr <<  "Usage: " << me 
       << " [-f <filename> | -m <sec>]  [-p <server port>] [-R <Hz>]\n";
  cerr << " [-t <VDIF | RAW | K5VSSP>] [-T <VDIF | RAW | K5VSSP>] [-w <sec>]\n"; 
  //  cerr << " [-r <Mbps>] [-s <size>] [-l <sec>] [-a dest host>] \n";
  cerr << " [-d <log path>] [-v <level>] \n";
  cerr << "-----------------------------\n";
  cerr << "  -f: sending file with SUDP packets."
       << " (if not specified, dummy data will be sent from memory.)\n";
  cerr << "  -m: sending dummy data for duration of <sec> .\n";
  //  cerr << "  -s: Packet size(mtu). default=" <<  DEFAULT_VDIF_FRAMESIZE + 8 << endl;
  cerr << "  -t: Data Frame Type For Output (default: VDIF) \n" ;
  cerr << "  -T: Data Frame Type for Input  (default: K5/VSSP32F) \n" ;
  cerr << "  -w: Page Reading Wait time (default: -1 endless) \n" ;
  //  cerr << "  -r: Rate Mbps (default: 100) \n" ;
  cerr << "  -p: Control Port of this host (default " << DEFAULT_SUDP_PORT
       << ") \n";
  //  cerr << "  -a: Remote hostname (default localhost)\n";
  //  cerr << "  -l: Length of transmission (default 10 sec)\n";
  cerr << "  -R: Rate Adjust Frequency (default 100 Hz)\n";
  cerr << "  -d: log directory path (default /tmp )\n";
  cerr << "  -v: verbose level, 0--4 (default 0).\n";
  cerr << " Version: " << version_str << endl;
  cerr << "  ----------- $id$ -------" << endl;
  exit(1); 
}

int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  string fname;
  string hostname("localhost");
  int srv_port(DEFAULT_CTRL_PORT);
  //  bool mflag(false); ///< Flag of memory data
  int mtu(DEFAULT_VDIF_FRAMESIZE +8);  
  int Mbps(100); // Mbps 
  int length(10); // sec
  int Rate_adj_f = 100; // (Hz)
  VLBI_Frame* out_vlbi_frame(0);
  VLBI_Frame* in_vlbi_frame(0);
  //  DAS_Info das_inf;
  int input_wait=-1;
  int verbose_level = 0;
  string logdir = "/tmp";
  
  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hf:d:p:m:t:w:v:R:")) { 
    case 'h':   usage();     break;
    case 'f':   fname = optarg; break;
    case 'd':   logdir = optarg; break;
      //    case 's':   mtu = atoi(optarg); break;
    case 't':   
      if(strncpy(optarg,"VDIF",4)==0){out_vlbi_frame = new VDIF_Frame; } 
      if(strncpy(optarg,"VSSP",4)==0){in_vlbi_frame = new K5VSSP32_Frame; } 
      break;
    case 'w':   input_wait = atoi(optarg); break;
      //    case 'r':   Mbps = atoi(optarg); break;
    case 'R':   Rate_adj_f = atoi(optarg); break;
    case 'm':   length  = atoi(optarg); break;
      //    case 'a':   hostname = optarg; break;
    case 'p':   srv_port  = atoi(optarg); break;
    case 'v':   verbose_level = atoi(optarg); break;
      //    case 'l':   length  = atoi(optarg); break;
    case '?':    usage(); break;
    }
  }
  logdir += "/";

//   if(mflag == false && fname.empty()){
//     cerr << "Error: Filename is not specified.\n";
//     usage();
//   }
  if( in_vlbi_frame == 0 )   in_vlbi_frame = new K5VSSP32_Frame;
  if( out_vlbi_frame == 0 ) out_vlbi_frame = new VDIF_Frame;

  
  
  string info_file = logdir + me + "_info.txt";
  string debug_file = logdir + me + "_dbg.txt";
  string error_file = logdir + me +"_error.txt";
  Logsystem::info_logger()->set_stream(info_file);
  Logsystem::debug_logger()->set_stream(debug_file);
  Logsystem::error_logger()->set_stream(error_file);
  Logsystem::debug_logger()->enable_logging();
  Logsystem::info_logger()->enable_logging();
  Logsystem::error_logger()->enable_logging();

  switch(verbose_level){
  case 0:
    Logsystem::debug_logger()->enable_logging(Logger::MINIMUM); 
    break;
  case 1:
    Logsystem::debug_logger()->enable_logging(Logger::SMALL); 
    break;
  case 2:
    Logsystem::debug_logger()->enable_logging(Logger::MEDIUM); 
    break;
  case 3:
    Logsystem::debug_logger()->enable_logging(Logger::DETAIL); 
    break;
  case 4:
    Logsystem::debug_logger()->enable_logging(Logger::ALL); 
    break;
  default:
    cerr << "Warning: Verbose level specified was out of range. Meduam is used instead.\n";
    Logsystem::debug_logger()->enable_logging(Logger::MEDIUM); 
    break;
  }
  
  /** Simple UDP Protocl for transmission */
  SUDPProto  sudp(mtu);
  sudp.Set_OprMode(TransInfo::TX); // Transmission mode 
  sudp.Set_Adj_Freq(Rate_adj_f);
  sudp.Set_Speed(Mbps,true); // Setti100 Mbps
  sudp.Disp_TxRate(); 
  sudp.Set_VlbiFrame(out_vlbi_frame); // Set Output Frame 
  

  /** Transport Control object */ 
  TransCtl transctl_proto;
  transctl_proto.Set_CtrlPort(srv_port);
  //  out_vlbi_frame->Set_DasInfo(&das_inf); 
  //  in_vlbi_frame->Set_DasInfo(&das_inf); 
  
  //  transctl_proto.Set_DasInfo(&das_inf); // Set DasInfo
  transctl_proto.Set_ApplProto(&sudp); // Set TransInfo
  
  fprintf(stderr,"Server- Port: %d  MTU= %d \n",  srv_port, mtu);
  fprintf(stderr,"Initial Rate : %d Mbps  For %d sec, Rate Adjust by %d cycle\n",
	  Mbps, length, Rate_adj_f);
  /***************************************/

  /** Data Input Device/File */
  InDev_Model *input_mdl;
  if(fname.empty()){ // sending test data from memory
    input_mdl = new InMem_Model(length);
    fprintf(stderr,"Data source: Memory   ") ;
  }else{   // sending from file.
    input_mdl = new InFile_Model;
    fprintf(stderr,"Data source : File(%s)  ", fname.c_str()); 
  }    
  //  input_mdl->Set_PageWait(input_wait);
  input_mdl->Set_VlbiFrame(in_vlbi_frame);
  ((InFile_Model*)input_mdl)->Set_infile(fname.c_str());  

  transctl_proto.Set_InputDev(input_mdl); // Set Input device 
  if(! transctl_proto.SetupCheck()) exit(1);
  //  input_mdl->Ready();
  //  input_mdl->start(); // reading data Started 

  //  signal(SIGINT,SIG_IGN);

  transctl_proto.start(); // Transaction server starting
  
//   char sRecvBuf[256];
//   while(transctl_proto.Status() == Thread::RUNNING ){
//     fprintf(stderr,"Terminate by \"QUIT\" \n");
//     scanf("%s", sRecvBuf);
//     StrUpper(sRecvBuf,strlen(sRecvBuf));
//     if(strncmp(sRecvBuf,"QUIT",4)==0){
//       break;
//     }
//   }
  
  cerr << "TransCtrl Thread ...";
  transctl_proto.join();
  cerr << "joined\n";

  sudp.Disp_RateOff();  

  /** Memory Release */
  if(in_vlbi_frame!=0) delete in_vlbi_frame;
  if(out_vlbi_frame!=0) delete out_vlbi_frame;
  if(input_mdl!=0) delete input_mdl;
  /****************************/

  if(Logsystem::Test_Logsys(Logsystem::DEBUG)) 
    Logsystem::debug_logger()->close_stream();
  if(Logsystem::Test_Logsys(Logsystem::INFO))   
    Logsystem::info_logger()->close_stream();
}


