
/** @file test_K5VSSP is test reading K5VSSP32 data 
 *  @author M.Sekido
 * @Version $Rev: 208 $
 * @Date  $Date: 2016-09-30 08:33:01 +0000 (Fri, 30 Sep 2016) $
 * $Id: test_Mk5B.cpp 208 2016-09-30 08:33:01Z vlbi $ 
 */

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <string>
#include "VLBI_Header.hpp"
using namespace std;

char *me;
const char* version_str = "$Rev: 208 $ updated $Date: 2016-09-30 08:33:01 +0000 (Fri, 30 Sep 2016) $ ";

void usage(){
  cerr << "Usage: " << me << " -f <filename> \n";
  //  cerr << "  -d <n>: Deltailed output level (default: 0)\n";
  cerr << " Version: " << version_str << endl;
  exit(1);
}

int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */
  int k=0;
  string fname;
  Mk5B_Header   mk5b_hdr;

  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hf:")) { 
    case 'h':  usage(); break;
    case 'f': fname = optarg; break;
      //    case 'd': mk5b_hdr.Set_dbg(atoi(optarg)); break;
    case '?' : /* Error:  character not in list */ 
      fprintf(stderr, /* Yes */ 
	      "%s WARNING:  Unknown option on command line \n", me); 
      break; 
    case ':' : /* Error:  missing parameter */ 
      fprintf(stderr, "%s WARNING:  Missing number on command line \n", me); 
      usage();
      break; 
    }
  }

  if(fname.empty()) usage();
  
  const int bsize = 32000;
  vector <uint8_t> buf(bsize);
  
  cerr << "size of buf=" << buf.size() << endl;

  FILE *fp = fopen(fname.c_str(),"r");
  if(fp ==NULL){
    cerr << "Error inn opening file: " << fname << endl;
    exit(1);
  }
  
  fprintf(stderr, "Mk5B headsize=%d\n", mk5b_hdr.Headsize());

  int32_t next_hdr_pos(0);
  int32_t total_byte=0;

  if( fread(&buf[0],bsize,1,fp) !=1){
    cerr << "Error: Failed to read data\n"; exit(1);
  }
  total_byte += bsize;
  int32_t ofst(0);

  while(!feof(fp) && ofst >=0){
    int pos = next_hdr_pos % bsize;
    while(next_hdr_pos< total_byte){
      ofst = mk5b_hdr.Hd_find(&buf[pos],bsize-pos);
      if(ofst <0 ){
	cerr << "Error faile to fined K5-VSSP header \n";
	break;
      }

      mk5b_hdr.Print_hd();
      uint32_t frame_size = mk5b_hdr.Framesize();
      next_hdr_pos += ofst + frame_size;
    }

    while(total_byte < next_hdr_pos){
      if( fread(&buf[0],bsize,1,fp) != 1){
	if(feof(fp)){
	  cerr << "End of File\n"; break;
	}else{
	  cerr << "Error: Failed to read data\n"; break;
	}
      }
      total_byte += bsize;
    }
    
  }
  
  fclose(fp);
}
