#include <iostream>
#include <vector>
#include <string>

#include <cstdio>
#include <cstring>
#include <cstdlib>
using namespace std;

#include "VTProto.hpp"
//#include "strngutl.hpp"
//#include "VLBI_Frame.hpp"
#include "VDIF_Frame.hpp"
#include "InMem_mdl.hpp"
//#include "Trans_ctl.hpp"
#include "Dumper.hpp" 
#include "StaticVals.hpp"
#include "conf_utl.hpp"
#include "main_cmn.hpp"

/** General Configulation data */
Config_set vdif_conf;

extern "C"{
#include <signal.h>
}

const char* version_str = "$Rev$ updated $Date$ ";

void usage(){
  cerr <<  "Usage: " << me 
       << " [-m <sec>]  [-p <server port>] [-R <Hz>] [-i <input mode>] [-a <dest host>]\n";
  cerr << " [-w <sec>]\n"; 
  //  cerr << " [-r <Mbps>] [-s <size>] [-l <sec>]  \n";
  cerr << " [-d <log path>] [-v <level>] \n";
  cerr << "-----------------------------\n";
  cerr << "  -m: sending dummy data for duration of <sec> .\n";
  cerr << "  -i: <'xxMsps/xbit/xxch'>: Input  Data mode\n";
  cerr << "  -a: Remote hostname (default localhost)\n";
  //  cerr << "  -s: Packet size(mtu). default=" <<  DEFAULT_VDIF_FRAMESIZE + 8 << endl;
  //  cerr << "  -w: Page Reading Wait time (default: -1 endless) \n" ;
  //  cerr << "  -r: Rate Mbps (default: 100) \n" ;
  //  cerr << "  -p: Control Port of this host (default " << DEFAULT_SUDP_PORT   << ") \n";
  //  cerr << "  -l: Length of transmission (default 10 sec)\n";
  cerr << "  -R: Rate Adjust Frequency (default 100 Hz)\n";
  cerr << "  -d: log directory path (default /tmp )\n";
  cerr << "  -v: verbose level, 0--4 (default 0).\n";
  cerr << " Version: " << version_str << endl;
  cerr << "  ----------- $id$ -------" << endl;
  exit(1); 
}

int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  string hostname("localhost");
  string Stname="Kashim34", Stid="kb";
  //  int srv_port(DEFAULT_CTRL_PORT);
  int mtu(DEFAULT_VDIF_FRAMESIZE +8);  
  int Mbps(32000000); // Mbps 
  int length(86400); // sec
  int Rate_adj_f = 100; // (Hz)
  DAS_Info das_inf( 32000000, 16, 2 ); //32000000
  string inMode;
  //  int input_wait=-1;
  int verbose_level = 0;
  string logdir = "/tmp";
  //int ndiv=1; // Number of frames in 1sec in VDIF output file
  //bool passive_mode = false;
	
  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "h:d:p:m:w:v:R:i:a:")) { 
    case 'h':   usage();     break;
    case 'i':   inMode = optarg; break;
    case 'd':   logdir = optarg; break;
      //    case 's':   mtu = atoi(optarg); break;
      //    case 'w':   input_wait = atoi(optarg); break;
      //    case 'r':   Mbps = atoi(optarg); break;
    case 'R':   Rate_adj_f = atoi(optarg); break;
    case 'm':   length  = atoi(optarg); break;
    case 'a':   hostname = optarg; break;
      //    case 'p':   srv_port  = atoi(optarg); break;
    case 'v':   verbose_level = atoi(optarg); break;
      //    case 'l':   length  = atoi(optarg); break;
    case '?':    usage(); break;
    }
  }
  logdir += "/";

  string info_file = logdir + me + "_info.txt";
  string debug_file = logdir + me + "_dbg.txt";
  string error_file = logdir + me +"_error.txt";
  Logsystem::info_logger()->set_stream(info_file);
  Logsystem::debug_logger()->set_stream(debug_file);
  Logsystem::error_logger()->set_stream(error_file);
  Logsystem::debug_logger()->enable_logging();
  Logsystem::info_logger()->enable_logging();
  Logsystem::error_logger()->enable_logging();

  switch(verbose_level){
  case 0:
    Logsystem::debug_logger()->enable_logging(Logger::MINIMUM); 
    break;
  case 1:
    Logsystem::debug_logger()->enable_logging(Logger::SMALL); 
    break;
  case 2:
    Logsystem::debug_logger()->enable_logging(Logger::MEDIUM); 
    break;
  case 3:
    Logsystem::debug_logger()->enable_logging(Logger::DETAIL); 
    break;
  case 4:
    Logsystem::debug_logger()->enable_logging(Logger::ALL); 
    break;
  default:
    cerr << "Warning: Verbose level specified was out of range. Meduam is used instead.\n";
    Logsystem::debug_logger()->enable_logging(Logger::MEDIUM); 
    break;
  }
	
  das_inf.SetMode(inMode);
  cerr << "Input Mode=" << das_inf.Mode() << endl;
  VLBI_Frame* in_vlbi_frame(0);
  VDIF_Frame vdif_frame;
  vdif_frame.Set_DasInfo(das_inf);

  VTProto vtp(mtu);
  /** Simple VTP Protocl for transmission */
  vtp.Set_OprMode(TransInfo::TX); // Transmission mode 
  vtp.Set_Adj_Freq(Rate_adj_f);
  vtp.Set_Speed(Mbps,true); // Setti100 Mbps
  vtp.Disp_TxRate(); 
  vtp.Set_Host(hostname.c_str());
  //vdif_frame.Set_Nframe_sec( ndiv ); 
  vtp.Set_VlbiFrame(&vdif_frame); // Set Output Frame 
  vtp.Set_mtu( mtu );
  vtp.Set_AckSend_ONOFF(false);
  vtp.Set_Ack_ONOFF(true);
  vtp.Set_Applprotocol(VLBI_Proto::VTP_Proto);
  /***************************************/

 if( in_vlbi_frame == 0 )   in_vlbi_frame = new VDIF_Frame;
  /** Data Input Device/File */
  //InMem_Model inmem;
  InDev_Model *input_mdl;
  input_mdl = new InMem_Model(length);

  //inmem.Set_protocol(&vtp);
  //input_mdl->Set_protocol(&vtp);
  //inmem.Set_VlbiFrame(&vdif_frame);
  in_vlbi_frame->Set_DasInfo( das_inf );
  input_mdl->Set_VlbiFrame(in_vlbi_frame);
  fprintf(stderr,"Data source: Memory   \n") ;
  input_mdl->Set_StnameID(Stname.c_str(),Stid.c_str());
	
  input_mdl->Set_DasInfo(das_inf);
	
  //if( inmem.Ready() < 0 ){
  if( input_mdl->Ready() < 0 ){
  	cerr << "Error in innet.Ready()\n";
    exit(1);
  }
  //inmem.start(); // reading data Started 
  input_mdl->start(); // reading data Started 
  sleep(1);
  	
  Dumper dumper; 

//  if(passive_mode)
    dumper.SetDasMode(VirDumper::IN2OUT);
//  else
//    dumper.SetDasMode(VirDumper::USR_OUT_SET);

  dumper.Set_Nout(1); 
  //if(dumper.Set_IO(&vtp, &inmem) <0){
  if(dumper.Set_IO(&vtp, input_mdl) <0){
  	cerr << "Error Dumper::Set_IO()\n";
    return -1;
  }

  if( dumper.Ready() ){
    cerr << " Dump Starting." << endl;
    dumper.start();
  }else{
    cerr << " Dump::Ready() Error." << endl;
  }

  dumper.join(); // Waiting for the dumper-thread returns.
    
  vtp.Disp_RateOff();  

  if(input_mdl!=0) delete input_mdl;

  /****************************/

  if(Logsystem::Test_Logsys(Logsystem::DEBUG)) 
    Logsystem::debug_logger()->close_stream();
  if(Logsystem::Test_Logsys(Logsystem::INFO))   
    Logsystem::info_logger()->close_stream();
}


