/** 
 * @Version $Rev: 220 $
 * @Author M.Sekido
 * @Date  $Date: 2017-05-25 05:06:42 +0000 (Thu, 25 May 2017) $
 * $Id: Mk5B_Frame.cpp 220 2017-05-25 05:06:42Z vlbi $ 
 */

#include "Mk5B_Frame.hpp"


int Mk5B_Frame::Set_header(const uint8_t* buf, const int ifrm){

  m5hdr.Set_header(data_epoch, sec_per_frame); //add fsec miya 2017/5/9
  unsigned char crcdata[6];

  crcdata[0] = m5hdr.mjd2<<4|m5hdr.mjd1;
  crcdata[1] = m5hdr.mjd0<<4|m5hdr.s4;
  crcdata[2] = m5hdr.s3<<4  |m5hdr.s2;
  crcdata[3] = m5hdr.s1<<4  |m5hdr.s0;
 
  crcdata[4] = m5hdr.ms4<<4|m5hdr.ms3;
  crcdata[5] = m5hdr.ms2<<4|m5hdr.ms1;
  m5hdr.crc = pCRC_mdl->Cmp_CRC(crcdata,6);
  if( ifrm >=0)   m5hdr.iframe = ifrm;
  memcpy((void*)buf,&m5hdr, Mk5B_Hdr_size);
  m5hdr.iframe++;

  //  memcpy((void*)buf,&m5hdr, sizeof(m5hdr));
  return sizeof(m5hdr);
}

// void Mk5B_Frame::Set_mark5header( uint32_t *header, const time_t epoch, 
// 				  const double sec, 
// 				  const uint16_t iblk){
//   header[0]=0xABADDEED;
//   header[1]=0xF00F0000 | iblk;
//   header[2]=0x00000000;
//   header[3]=0x00000000;
//   //  uint16_t *crc = (unsigned short*)&header[3];


//   //  char time_code[32];
//   //  strftime(time_code,sizeof(time_code),"%Y/%j %H:%M:%S",gmtime(&epoch));
//   // strptime(time_code,"%Y/%j %H:%M:%S",&tm);
//   //  double mjd = tm2mjd(tm);
//   //  fprintf(stderr, "MJD1 = %.5lf ",mjd);

//   struct tm tm;
//   gmtime_r(&epoch,&tm);
//   double mjd=tm2mjd(tm);
//   //  fprintf(stderr, "MJD2 = %.5lf \n",mjd);


//   int intmjd = (int)floor(mjd);
//   int secmjd = (int)floor((mjd-intmjd)*86400+5e-7);  // Avoid round off errors
  
//   //  fprintf(stderr,"Writing: %03d/%04d +%6.4lf sec \n", intmjd);
//   // Word 2 VLBA BCD time code 'JJJSSSSS'. Only needs to be updated
//   // once a second

//   unsigned char j1,j2,j3,s1,s2,s3,s4,s5;  
//   j1 = (intmjd%1000/100);
//   j2 = ((intmjd%100)/10);
//   j3 = (intmjd%10);
  
  
//   s1 = (secmjd/10000)&0xF;
//   s2 = ((secmjd%10000)/1000)&0xF;
//   s3 = ((secmjd%1000)/100)&0xF;
//   s4 = ((secmjd%100)/10)&0xF;
//   s5 = (secmjd%10)&0xF;
  
//   header[2]  = j1<<28 | j2<<24 | j3<<20 | s1<<16 | s2<<12 | s3<<8 | s4<<4 | s5;
  
//   unsigned char crcdata[6];
//   crcdata[0] = j1<<4|j2;
//   crcdata[1] = j3<<4|s1;
//   crcdata[2] = s2<<4|s3;
//   crcdata[3] = s4<<4|s5;
  
//   // MJD sec fraction
//   int fracsec = (int)(sec*10000);
  
//   s1 = fracsec/1000;
//   s2 = (fracsec%1000)/100;
//   s3 = (fracsec%100)/10;
//   s4 = fracsec%10;
//   header[3] = s1<<28 | s2<<24 | s3<<20 | s4<<16;
  
//   crcdata[4] = s1<<4|s2;
//   crcdata[5] = s3<<4|s4;

//   uint16_t crc = pCRC_mdl->Cmp_CRC(crcdata,6);
//   // *crc = reversebits16(crcc(crcdata, 48, 040003, 16));
//   //  printf("Header = {%08x,%08x,%08x,%08x}\n",header[0],header[1],header[2],header[3]);

//   header[3] |= (crc & 0xffff);
//   //  printf("Header = {%08x,%08x,%08x,%08x}\n",header[0],header[1],header[2],header[3]);
// }  

// const long Mk5B_Frame::Read_one_sec_hdr(vector <Mk5B_Hdr>& hdrarr){

//   if(fp==0){
//     cerr << "E Failed Mk5B_Frame::Test_Read(): File is not opened yet\n";
//     return -1;
//   }
//   hdrarr.resize(2);
//   Mk5B_Hdr hdr;
//   if(fread((void*)&hdr,sizeof(hdr),1,fp)<0 || ! hdr.HD_Match()){
//     cerr << "E: Mk5B_Frame::Read_one_sec_hdr(): Failed to get header.\n;";
//     return -1;
//   };
//   int sod0 = (int)hdr.Sod();
//   int iframe0 = hdr.iFrame();
// }

const long Mk5B_Frame::Read_every_sec_hdr(vector <Mk5B_Hdr>& hdrarr,
					  const size_t nsec){
  char exceptionMessage[1024];

  try{
    if(fp==0){
      //      cerr << "E Failed Mk5B_Frame::Test_Read(): File is not opened yet\n";
      strcpy(exceptionMessage, "File is not opened yet");
      throw exceptionMessage;
    }

    if(nframe_sec ==0 && (! Fix_nFrame_sec() )){
      strcpy(exceptionMessage, "Failed to fix nframe_sec");
      throw exceptionMessage;
    }


    char buf[framesize];
    Mk5B_Hdr *phdr = (Mk5B_Hdr *)buf;
    hdrarr.resize(0);
    int icnt(0);
    for(icnt=0; icnt < (int)nsec; icnt++){
      if(fread( (void*)&buf[0], framesize, 1, fp)<1){
	sprintf(exceptionMessage, "Failed to read iframe(%d)", icnt);
	throw exceptionMessage;
      }
      hdrarr.push_back(*phdr);
      fseek(fp,(nframe_sec - 1)*framesize,SEEK_CUR);      
    }
      

//     // Search for the next 1sec
//     int sec_count(0);
//     //  long frm_intvl(0);
//     long frm_count(0);
//     char buf[256];
//     while( nsec == 0 || hdrarr.size() <= nsec ){
//       if(!Mk5B_Hdr_find(fp, offset, hdr)) break;
//       fseek(fp,framesize,SEEK_CUR);
//       frm_count++;
//       if(frm_count % 100 == 0){
// 	sprintf(buf,"Frame count=%ld \r",frm_count);
// 	write(1,buf,strlen(buf));
//       }
//       if( (int)(sec_dat[1] = hdr.Sod()) != (int)sec_dat[0]){
// 	hdrarr.push_back(hdr);
// 	//       if(sec_dat[1] - sec_dat[0] == 1)
// 	// 	frm_intvl = hdrarr[sec_count].iFrame() - hdr.iFrame();
// 	sec_dat[0] = sec_dat[1];
// 	sec_count++;    
// 	write(1,"\n",strlen("\n"));
//       };
//     }
//     //  frm_count --; // Reduce for extracounting at that last
//     //  if(frm_intvl >0) nframe_sec = frm_intvl;
//     nframe_sec = frm_count/sec_count;

//     // Set Srate for reference for non-NICT version
//     long datsize = Framesize() - Headsize();
//     DAS_Info das = DAS_Inf();
//     int nch = das.Nch();
//     int qbit = das.Qbit();
//     int32_t srate_ksps = frm_count * datsize * 8/sec_count/qbit/nch/1000;

//     if( DAS_Inf().Srate()/1000 != srate_ksps ){ // Das info (srate) updated.
//       fprintf(stderr,"W Srate info(%d Msps) updated -> %d Msps\n",
// 	      DAS_Inf().Srate()/1000000, srate_ksps/1000);
//       das.Set_Srate(srate_ksps*1000);
//       Set_DasInfo(das);
//     }
  
    return icnt;
  }// end of try

//   catch(string warn){
//     cerr << "W:Mk5B_Frame::Read_every_sec_hdr():" << warn << endl;
//   }

  catch(char* str){
    cerr << "E:Mk5B_Frame::Read_every_sec_hdr():" << str << endl;
    return -1;
  }
  
}

/** Read Header data 
 * hdrarr : header array for returning the results.
 * nfrm : Number of frames to be read.
 * rewind: Rewind option to set the initial File Pointer position 
 **/
const long Mk5B_Frame::Read_every_frame_hdr(vector <Mk5B_Hdr>& hdrarr,
					    const size_t nfrm, const bool rewind_flag){

  vector <char> buf(framesize);
  char exception_Message[512];
  size_t rdsize(0);
  hdrarr.resize(0);
  try{
    if(fp==0 || fread( (void*)&buf[0], framesize, 1, fp) < 1){
      char errmsg[512];
      sprintf(exception_Message," File is not opened yet(%s)\n",
	      strerror_r(errno, errmsg,512));
      throw exception_Message;
      //      return -1;
    }
    char* ptr = &buf[0];
    Mk5B_Hdr *phdr = (Mk5B_Hdr*) &buf[0];
    
    rdsize += framesize;

    if( !phdr->HD_Match()){
      Mk5B_Hdr hdr;
      long offset(0);
      if( Mk5B_Hdr_find(fp, offset, hdr)){
	rdsize += offset;
	if(fread( (void*)&buf[0], framesize, 1, fp) == 1) rdsize += framesize;
      }else{
	rdsize += offset;
	if(rewind_flag) fseek(fp,-rdsize, SEEK_CUR);
	sprintf(exception_Message," Header not found");
	throw exception_Message;
      }
    }
    hdrarr.push_back(*phdr);

    buf.resize(framesize*(nfrm-1));
    phdr = (Mk5B_Hdr*) &buf[0];
    ptr = &buf[0];
    long nrd_frm = fread( (void*)&buf[0], framesize, (nfrm-1), fp);
    rdsize += nrd_frm * framesize;
    if(rewind_flag) fseek(fp,-rdsize, SEEK_CUR);    
    long ifrm_cnt=1;
    for(int i = nrd_frm; i> 0; i--){
      if(!phdr->HD_Match()){
	sprintf(exception_Message," Header not found at iframe(%ld)", ifrm_cnt);
	throw exception_Message;
      }
      hdrarr.push_back(*phdr);
      ifrm_cnt++; 
      ptr += framesize;
      phdr = (Mk5B_Hdr*) ptr;
    }
    
    return ifrm_cnt;
  }// End try

  catch( char* ){
    cerr << "E:Mk5B_Frame::Read_every_frame_hdr():" << exception_Message << endl;
    return -1;
  }

}

const long Mk5B_Frame::Read_sec( vector <uint8_t>& rawbuf, const int sec){
  vector <Mk5B_Hdr> hdrarr;
//   if( !Tested() ||  Read_every_sec_hdr(hdrarr, sec) <=0){
//       cerr << "  E Failed to Mk5B_Frame::Read_sec()" << endl;
//       return -1;
//   };

  long pktsize = hdrarr[0].Framesize();
  size_t bufsize = nframe_sec* pktsize * sec;
  if(rawbuf.size() != bufsize)  rawbuf.resize(bufsize);
  long cnt = fread( &rawbuf[0], bufsize, 1, fp);

  long nframe = cnt*bufsize/pktsize;
  return nframe;
}

const long Mk5B_Frame::Read_frame( vector <Mk5B_Hdr>& hdrarr, 
				   vector <uint8_t>& datbuf,
				   const int nframe){
  
  // Set Pointers 
  Mk5B_Hdr *phdr =  (Mk5B_Hdr*)&workbuf[0];

  if(  hdrarr.size() < 1 )  hdrarr.resize(nframe);
  size_t hdsize =  hdrarr[0].Headsize();
  long framesize = hdrarr[0].Framesize(); 
  size_t bufsize = framesize * nframe;
  long datsize = framesize - hdsize;
  if(  workbuf.size() < bufsize){
    workbuf.resize(bufsize); 
    phdr =  (Mk5B_Hdr*)&workbuf[0];
  }
  
  size_t rframe = fread( &workbuf[0], framesize, nframe,fp);
  if(rframe < 1){
    perror("E Mk5B_Frame::Read_frame():");
    return -1;
  }
  if(  hdrarr.size() != rframe)  hdrarr.resize(rframe);
  size_t datbufsize = datsize * rframe;  
  if(  datbuf.size() != datbufsize)  datbuf.resize(datbufsize);
  
  vector <uint8_t>::iterator ptr = workbuf.begin() + hdsize;
  vector <uint8_t>::iterator ptrd = datbuf.begin();
  m5hdr = *phdr; // Set first data to the common hdr-data in the frame

  for(int i=0; i< (int)rframe; i++){
    hdrarr[i] = *phdr;
    memcpy(&(*ptrd), &(*ptr), datsize);
    ptrd += datsize; ptr += datsize;  
    phdr = (Mk5B_Hdr*)&(*ptr); ptr += hdsize;
  }

  Set_Epoch(m5hdr.Epoch());
  
  return rframe;
}
  

