/**  
 * VTProto.hpp This file contains class interface of VTP Protocol
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (水, 03  7月 2013) $
 * $Id: VTProto.hpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#ifndef __VTPROTO_HPP__
#define __VTPROTO_HPP__

#include <UDPSocket.h>
#include <vector>
#include <string>
#include <cstring>
#include <cstdio>
using namespace std;

#include "Logsystem.hpp"
#include "VLBI_Proto.hpp"
#include "VDIF_Header.hpp"
#include "algorithm"
#include "light_utl.hpp"
#include "DataPage.hpp"

extern "C" {
#include <sys/socket.h>
#include <sys/time.h>
#include <stdint.h>
}

//const int DEFAULT_SUDP_MTU = 1008;

struct VtpHdr {
///  enum CMD_t {INIT, VALID, END};
  uint64_t seq;
///  uint32_t seq;
///  uint8_t cmd;
///  uint8_t  data[3];
public:
	///SudpHdr():seq(0),cmd(INIT){ memset(data,0,3);}
	VtpHdr():seq(0){;}
  uint64_t operator++(int z){seq ++; return seq;}
///  uint64_t operator++(int z){seq ++; return seq;}
  void ResetSeq(){seq=0;}
///  void SetInit(){cmd = (uint8_t)INIT;}
///  void SetValid(){cmd = (uint8_t)VALID;}
///  void SetEND(){cmd = (uint8_t)END;}
///  void SetFramesize(const int32_t sz);
///  const int32_t Framesize() const;
///  const uint32_t Seq() const {return seq;}
	const uint64_t Seq() const {return seq;}
///  const CMD_t Cmd() const {return (CMD_t)cmd;}
  //  void SetHdr(void* p){memcpy(p,(void*)this,SUDP_HEAD_SIZE);}
};

///inline void SudpHdr::SetFramesize(const int32_t sz){
///  data[0] = sz & 0xff;
///  data[1] = (sz >> 8) & 0xff;
///  data[2] = (sz >> 16 )& 0xff;
///}

///inline const int32_t SudpHdr::Framesize() const{
///  int32_t ret = (data[2] << 16) | (data[1] << 8) | data[0];
///  return ret;
///}

///double tim(void);
///void my_usleep(double usec);
///void my_usleep2(const ulong usec,const struct timeval tm1);
///void my_usleep2(const ulong usec);
///const ulong Tdif(const struct timeval tm1, const struct timeval tm2);

/**
 * SUDPPorto is Simple UDP (serial number + UDP) protocol class.
 * This class is for  a simple protcol just adding 32bit-size sequensial 
 * numbering (seq) to manage the packet loss and duplication.
 *  - Input: File pointer or Buffer memory or Netowrk
 *  - Output: network or Buffer 
 *  @param seq : sequential number start from 0;
 * \author $Author: vlbi $
 * \Version $Revision: 56 $
 * \date $Date: 2013-07-03 20:40:14 +0900 (水, 03  7月 2013) $
 */
class VTProto:public VLBI_Proto{
  /** Sequential Number of UDP packet*/
  //  int64_t  seq; 
  VtpHdr  vtphdr;
  //  ExtSubCh extch;
  VTProto(VTProto& sp){;} ///< Copy Constructor, prohibited.
public:
  /** Default Constructor */
  VTProto():
    VLBI_Proto(VTP_Proto, UDP_IO){ }
  /** Constructor */
  VTProto(const int m){Set_mtu(m);}  
  /// Destructor 
  ~VTProto(){
    cerr << "VTProto:: Destructor runs.\n";
    if(rate_meas.is_running()) rate_meas.stop();
    if(_sock !=0){ delete _sock; _sock=0;}
  }
  /**  Transmitting data from Buffer for 1sec 
   * @param t Current epoch in unix time
   * @param arr vector array data.
   */
  int32_t TX1sec(const time_t t, vector <uint8_t>& arr);
  /** Transmitting subset channels of 1sec of data 
   * @param t Current epoch in unix time
   * @param arr vector array of data.
   * @param das Input DAS mode information
   */
  int32_t TX1secSub(const time_t t, vector <uint8_t>& arr, const DAS_Info* das=0);
  /**  Transmitting data from Buffer for 1sec, pointer version 
   * @param t Current epoch in unix time
   * @param arr pointer to data array
   * @param ndat size of data array in bytes 
   */
  int32_t TX1sec(const time_t t,  const uint8_t* arr, const int32_t ndat);
  /** Transmitting subset of Observation mode for 1sec of data, pointer version 
   * @param t Current epoch in unix time
   * @param arr pointer to data array
   * @param ndat size of data array in bytes 
   * @param das Input DAS mode information
    */
  // TX1seczero add 2016/9/29 miya
  int32_t TX1seczero(const time_t t, const int32_t ndat){
    fprintf(stderr, "VTProto::TX1seczero() is not necessary,\n");
    return -1;
  }
  /** Transmitting subset of Observation mode for 1sec of data, pointer version 
   * @param t Current epoch in unix time
   * @param arr pointer to data array
   * @param ndat size of data array in bytes 
   * @param das Input DAS mode information
   */
  int32_t TX1secSub(const time_t t,  const uint8_t* arr, const int32_t ndat, const DAS_Info* das=0);
  // TX1secSubzero add 2016/9/29 miya
  int32_t TX1secSubzero(const time_t t,   const int32_t ndat, const DAS_Info* das=0){
    fprintf(stderr, "VTProto::TX1secSubzero() is not necessary,\n");
    return -1;
  }
  //   /** @doc Receiving data to Buffer for 1 sec */
  //   int32_t RX1sec(vector <uint8_t>& arr);
  /** Transmitting data from File
   * @param fp: File pointer as data source
   * @return size of data transmitted.
   */
  int32_t TX(FILE *fp);
  /**  Receiving data to File */
  //  int32_t RX(FILE *fp=NULL);
  /**  Receiving data to Buffer */
  int32_t RX_UDP(void* buf, const int32_t buf_size);
  ///int RX_SUDP(DataPage& dpage);
  int RX_VTP(DataPage& dpage);
	///const uint64_t RX_SUDP(FILE* fp);
  const uint64_t RX_VTP(FILE* fp);
	int RXvdif(DataPage& dpage);
  //  ApplProtocol_t ApplProtocol_Type()const {return SUDP_Proto;}
  int ApplProtoHeadSize()const{return VTP_HEAD_SIZE;}
  //  void Set_VlbiFrame(VLBI_Frame * vf);
  void Rate_Adj(const int32_t sent_bytes=-1);
  //  void Runflag_stop(){runflag = false; cerr << "runflag = false" << endl; }
  bool Tx_Ready(){ vtphdr.ResetSeq(); return VLBI_Proto::Tx_Ready(); }
  void FirstPacket(); ///< Senidng INIT cmd packet 
  void LastPacket();  ///< Senidng END cmd packet 
  void Finish();      ///< Reset EndEpoch and StopTimer
private:
  static const bool Send_Enable=true; // add miya 2015/6/23
  static const bool Ack_Wait_Call=true; // add miya 2015/6/23
};


inline void  VTProto::Rate_Adj(const int32_t sent_bytes){
  double wait = rate_meas.Rcmnd_wait();
  if(wait >0) {
    if( Logsystem::Test_Logsys(Logsystem::DEBUG, 4)){   
      Logsystem::debug_logger()->Tstamp() << "Rate_Adj(): sleep " << wait << " sec. ";
      *Logsystem::debug_logger()
	<<"Inst:" << rate_meas.Tx_Mbps()  << "Mbps (net:" << rate_meas.Tx_Pld_Mbps()
	<< "Mbps)/Target=" << rate_meas.TargetRate() << "Mbps\n";
      Logsystem::debug_logger()->Flush();
    }

#ifdef _MY_SLEEPVTP_
    my_usleep2_vtp((ulong)(wait*1.e6));
#else
    struct timespec req;
    req.tv_sec = (time_t)(wait);
    req.tv_nsec = (long)((wait - req.tv_sec)*1000000000);
    nanosleep(&req,0);
#endif
  }
}

// inline
// void VTProto::Set_Sparam(const int n, const int q, const uint32_t s){
//   vlbi_frame->Set_Sparam(n,q,s);
//   speed = vlbi_frame->DAS_Inf().Bytes_sec()/32;  // words/sec 
// }

// inline int VTProto::Set_VLBI_Frame(const VLBI_Frame_t ft){
  
//   if(vlbi_frame !=0) delete vlbi_frame;
//   switch(ft){
//   case MK5B_frm: vlbi_frame = new Mk5B_Frame; break;
//   case VDIF_frm: vlbi_frame = new VDIF_Frame; break;
//     //  case K5VSI_frm: vlbi_frame = new K5VSI_Frame; break;
//   default:
//     cerr << "Error in VTProto::Set_VLBI_Frame(): This frame type is not supported yet.\n";
//     return -1;
//     break;
//   }
//   return 0;
// }



inline double tim_vtp(void) {
  struct timeval tv;
  double t;

  gettimeofday(&tv, NULL);
  t = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
  return t;
}

inline void my_usleep_vtp(double usec) {
  double now, till;
  //  int n;
  /* The current time */
  now = tim_vtp();

  /* The time we are sleeping till */
  till = now+usec/1.0e6;
  /* and spin for the rest of the time */
  //  n = 0;
  while (now<till) {
    now = tim_vtp();
    //    n++;
  }
  
}

inline const ulong Tdif_vtp(const struct timeval tm1, const struct timeval tm2){
  ulong dt = (tm2.tv_sec - tm1.tv_sec)*1000000 + (tm2.tv_usec - tm1.tv_usec);
  return dt;
};

inline void my_usleep2_vtp(const ulong usec){
//   struct timeval tm2(tm1);
//   tm2.tv_usec += usec;
//   if(tm2.tv_usec > 1000000){tm2.tv_sec++; tm2.tv_usec -= 1000000;}
  
  struct timeval tm1;
  gettimeofday(&tm1,0);
  struct timeval tm;
  do{
    gettimeofday(&tm,0);
  }while( Tdif_vtp(tm1,tm) < usec);

}

inline void my_usleep2_vtp(const ulong usec,const struct timeval tm1){
//   struct timeval tm2(tm1);
//   tm2.tv_usec += usec;
//   if(tm2.tv_usec > 1000000){tm2.tv_sec++; tm2.tv_usec -= 1000000;}
  
  struct timeval tm;
  do{
    gettimeofday(&tm,0);
  }while( Tdif_vtp(tm1,tm) < usec);

}

#endif
