
/** Bit Opearation Utilities .
 * @Version $Rev: 144 $
 * @Author M.Sekido
 * @Date  $Date: 2014-10-17 09:41:42 +0900 (金, 17 10月 2014) $
 * $Id: bitutl.cpp 144 2014-10-17 00:41:42Z vlbi $ .
 */

#include "bitutl.hpp"

const float Decode::level_1bit[2] = {-1.,1.};
const float Decode::level_BINARY_2bit[4] = {-3., -1., 1., 3. };
const float Decode::level_RevBINARY_2bit[4] = {-3., 1., -1., 3. };
const float Decode::level_SignMag_2bit[4] = {1., -3., 1., 3. };
const float Decode::level_RevSignMag_2bit[4] = {-1., 1., -3., 3.};

const int Decode::Make_Table(const BitEnc_t enc_type){
  if(dasInfo ==0){
    cerr << "Error @Decode::Make_Table(): dasInfo is empty.\n";
    return -1;
  }
  try{
    switch(dasInfo->Qbit()){
    case 1: 	MkTbl1bit();	break;
    case 2: 	MkTbl2bit(enc_type);	break;
    case 4: 	MkTbl4bit();	break;
    case 8: 	MkTbl8bit();	break;
    default:	throw "Impossible Qbit"; break;
    }
  }

  catch(const char* msg){
    cerr << "Error @Decode::Make_Table(): " << msg << endl;
    return -1;
  }
  return 0;
}


const int Decode::MkTbl1bit(){
  if(dasInfo ==0){  
    cerr << "Error @Decode::MkTbl4_1(): dasInfo is empty.\n";
    return -1;  
  }
  for(int i=0; i<256; i++)  bzero(ftable[i], MAX_NCH*sizeof(float));
  //  int8_t *ptr;
  float  *ptrf;
  for(int i=0; i<256; i++){
//     ptr =  &table[i][0];
//     if(i & 0x01)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
//     if(i & 0x02)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
//     if(i & 0x04)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
//     if(i & 0x08)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
//     if(i & 0x10)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
//     if(i & 0x20)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
//     if(i & 0x40)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; ptr++;
//     if(i & 0x80)  *ptr=level_1bit[1]; else *ptr=level_1bit[0]; 
    
    ptrf = &ftable[i][0]; // For float 
    if(i & 0x01)  *ptrf=(float)level_1bit[1]; else *ptrf=(float)level_1bit[0]; ptrf++;
    if(i & 0x02)  *ptrf=(float)level_1bit[1]; else *ptrf=(float)level_1bit[0]; ptrf++;
    if(i & 0x04)  *ptrf=(float)level_1bit[1]; else *ptrf=(float)level_1bit[0]; ptrf++;
    if(i & 0x08)  *ptrf=(float)level_1bit[1]; else *ptrf=(float)level_1bit[0]; ptrf++;
    if(i & 0x10)  *ptrf=(float)level_1bit[1]; else *ptrf=(float)level_1bit[0]; ptrf++;
    if(i & 0x20)  *ptrf=(float)level_1bit[1]; else *ptrf=(float)level_1bit[0]; ptrf++;
    if(i & 0x40)  *ptrf=(float)level_1bit[1]; else *ptrf=(float)level_1bit[0]; ptrf++;
    if(i & 0x80)  *ptrf=(float)level_1bit[1]; else *ptrf=(float)level_1bit[0]; 
  }
  return 0;
}

const int Decode::MkTbl2bit(const BitEnc_t enc_type){
  if(dasInfo ==0){  
    cerr << "Error @Decode::MkTbl4_1(): dasInfo is empty.\n";
    return -1;  
  }
  //  for(int i=0; i<256; i++)  memset(table[i],0, MAX_NCH*sizeof(uint8_t));

  // uint8_t mask[4];
  // mask[0] = 0x03;
  // mask[1] = 0x03 << 2;
  // mask[2] = 0x03 << 4;
  // mask[3] = 0x03 << 6;

  switch(enc_type){
  case RevBINARY_2bit:
    for(int i=0; i<256; i++){
      for(int j=0; j<4; j++){
	int v = (i & (0x03 << (j*2))) >> (j*2);
	//	table[i][j]  =        level_RevBINARY_2bit[v];	
	ftable[i][j] = level_RevBINARY_2bit[v];	
      };
    }
    break;
  case SignMag_2bit:
    for(int i=0; i<256; i++){
      for(int j=0; j<4; j++){
	int v = (i & (0x03 << (j*2))) >> (j*2);
	//	table[i][j]  =        level_SignMag_2bit[v];	
	ftable[i][j] = level_SignMag_2bit[v];	
      };
    }
    break;
  case RevSgnMag_2bit:
    for(int i=0; i<256; i++){
      for(int j=0; j<4; j++){
	int v = (i & (0x03 << (j*2))) >> (j*2);
	//	table[i][j]  =        level_RevSgnMag_2bit[v];	
	ftable[i][j] = level_RevSignMag_2bit[v];	
      };
    }
    break;
  case BINARY_2bit:// This is natural and also be adopted by Makr5B and other VLBI terminal
  default:
    for(int i=0; i<256; i++){
      for(int j=0; j<4; j++){
	int v = (i & (0x03 << (j*2))) >> (j*2);
	//	table[i][j]  =        level_BINARY_2bit[v];	
	ftable[i][j] = level_BINARY_2bit[v];	
      };
    }
    break;
  }

  return 0;
}


const int Decode::MkTbl4bit(){
  if(dasInfo ==0){  
    cerr << "Error @Decode::MkTbl4_1(): dasInfo is empty.\n";
    return -1;  
  }
  for(int i=0; i<256; i++)  bzero( ftable[i], MAX_NCH*sizeof(float));
  for(int i=0; i<256; i++){
    ftable[i][0] = (float)((i & 0x0f) - 8);
    ftable[i][1] = (float)(((i & 0xf0) >> 4) - 8);
  }
  return 0;
}

const int  Decode::MkTbl8bit(){
  if(dasInfo ==0){  
    cerr << "Error @Decode::MkTbl4_1(): dasInfo is empty.\n";
    return -1;  
  }
  for(int i=0; i<256; i++)  bzero(ftable[i], MAX_NCH*sizeof(float));
  for(int i=0; i<256; i++){
    ftable[i][0] = (float)(i -128);
  }
  return 0;
}
